@@ -1,7 +1,3 @@
-# -*-mode: tcl; fill-column: 75; tab-width: 8; coding: iso-latin-1-unix -*-
-#
-#	$Id: Sample.tcl,v 1.3 2001/12/09 05:31:07 idiscovery Exp $
-#
 # Tix Demostration Program
 #
 # This sample program is structured in such a way so that it can be
@@ -30,7 +26,7 @@ proc RunSample {w} {
 if {![info exists tix_demo_running]} {
     wm withdraw .
     set w .demo
-    toplevel $w; wm transient $w ""
+    toplevel $w
     RunSample $w
     bind $w <Destroy> exit
 }
