@@ -1,6 +1,3 @@
-
-/*	$Id: tixImgXpm.c,v 1.5 2008/02/28 04:05:29 hobbs Exp $	*/
-
 /*
  * tixImgXpm.c --
  *
@@ -22,7 +19,7 @@
  */
 
 static int		ImgXpmCreate _ANSI_ARGS_((Tcl_Interp *interp,
-			    char *name, int argc, Tcl_Obj *CONST objv[],
+			    char *name, int argc, char **argv,
 			    Tk_ImageType *typePtr, Tk_ImageMaster master,
 			    ClientData *clientDataPtr));
 static ClientData	ImgXpmGet _ANSI_ARGS_((Tk_Window tkwin,
@@ -35,20 +32,20 @@ static void		ImgXpmFree _ANSI_ARGS_((ClientData clientData,
 			    Display *display));
 static void		ImgXpmDelete _ANSI_ARGS_((ClientData clientData));
 static int		ImgXpmCmd _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, int argc, CONST84 char **argv));
+			    Tcl_Interp *interp, int argc, char **argv));
 static void		ImgXpmCmdDeletedProc _ANSI_ARGS_((
 			    ClientData clientData));
 static void		ImgXpmConfigureInstance _ANSI_ARGS_((
 			    PixmapInstance *instancePtr));
 static int		ImgXpmConfigureMaster _ANSI_ARGS_((
-			    PixmapMaster *masterPtr, int argc, CONST84 char **argv,
+			    PixmapMaster *masterPtr, int argc, char **argv,
 			    int flags));
 static int		ImgXpmGetData _ANSI_ARGS_((Tcl_Interp *interp,
 			    PixmapMaster *masterPtr));
 static char ** 		ImgXpmGetDataFromFile _ANSI_ARGS_((Tcl_Interp * interp,
 			    char * string, int * numLines_return));
 static char ** 		ImgXpmGetDataFromId _ANSI_ARGS_((Tcl_Interp * interp,
-			    CONST84 char * id));
+			    char * id));
 static char ** 		ImgXpmGetDataFromString _ANSI_ARGS_((Tcl_Interp*interp,
 			    char * string, int * numLines_return));
 static void 		ImgXpmGetPixmapFromData _ANSI_ARGS_((
@@ -76,15 +73,13 @@ static Tk_ConfigSpec configSpecs[] = {
 };
 
 Tk_ImageType tixPixmapImageType = {
-    "pixmap",                       /* name */
-    ImgXpmCreate,                   /* createProc */
-    ImgXpmGet,                      /* getProc */
-    ImgXpmDisplay,                  /* displayProc */
-    ImgXpmFree,                     /* freeProc */
-    ImgXpmDelete,                   /* deleteProc */
-    NULL,                           /* postscriptProc (tk8.3 or later)*/
-    NULL,                           /* nextPtr */
-    NULL,                           /* reserved */
+    "pixmap",			/* name */
+    ImgXpmCreate,		/* createProc */
+    ImgXpmGet,			/* getProc */
+    ImgXpmDisplay,		/* displayProc */
+    ImgXpmFree,			/* freeProc */
+    ImgXpmDelete,		/* deleteProc */
+    (Tk_ImageType *) NULL	/* nextPtr */
 };
 
 /*
@@ -117,7 +112,7 @@ ImgXpmCreate(interp, name, argc, objv, typePtr, master, clientDataPtr)
 				 * image. */
     char *name;			/* Name to use for image. */
     int argc;			/* Number of arguments. */
-    Tcl_Obj *CONST objv[];	/* Argument strings for options (doesn't
+    char **objv;		/* Argument strings for options (doesn't
 				 * include image name or type). */
     Tk_ImageType *typePtr;	/* Pointer to our type record (not used). */
     Tk_ImageMaster master;	/* Token for image, to be used by us in
@@ -127,22 +122,17 @@ ImgXpmCreate(interp, name, argc, objv, typePtr, master, clientDataPtr)
 {
     PixmapMaster *masterPtr;
     int i;
-    CONST84 char *argvbuf[10];
-    CONST84 char **argv = argvbuf;
+    char *argvbuf[10];
+    char **argv = argvbuf;
 
     /*
      * Convert the objv arguments into string equivalent.
      */
     if (argc > 10) {
-	argv = (CONST84 char **) ckalloc(argc * sizeof(char *));
+	argv = (char **) ckalloc(argc * sizeof(char *));
     }
     for (i = 0; i < argc; i++) {
-        /*
-         * no need to free the value returned by Tcl_GetString. It's
-         * managed by Tcl's object system.
-         */
-
-	argv[i] = Tcl_GetString(objv[i]);
+	argv[i] = TixGetStringFromObj(objv[i], NULL);
     }
 
     masterPtr = (PixmapMaster *) ckalloc(sizeof(PixmapMaster));
@@ -183,7 +173,7 @@ ImgXpmCreate(interp, name, argc, objv, typePtr, master, clientDataPtr)
  *
  * Results:
  *	A standard Tcl return value.  If TCL_ERROR is returned then
- *	an error message is left in interp's result.
+ *	an error message is left in masterPtr->interp->result.
  *
  * Side effects:
  *	Existing instances of the image will be redisplayed to match
@@ -199,7 +189,7 @@ ImgXpmConfigureMaster(masterPtr, argc, argv, flags)
     PixmapMaster *masterPtr;	/* Pointer to data structure describing
 				 * overall pixmap image to (reconfigure). */
     int argc;			/* Number of entries in argv. */
-    CONST84 char **argv;	/* Pairs of configuration options for image. */
+    char **argv;		/* Pairs of configuration options for image. */
     int flags;			/* Flags to pass to Tk_ConfigureWidget,
 				 * such as TK_CONFIG_ARGV_ONLY. */
 {
@@ -287,9 +277,9 @@ ImgXpmGetData(interp, masterPtr)
     PixmapMaster *masterPtr;
 {
     char ** data = NULL;
-    int  isAllocated = 0;	/* do we need to free "data"? */
+    int  isAllocated;			/* do we need to free "data"? */
     int listArgc;
-    CONST84 char ** listArgv = NULL;
+    char ** listArgv = NULL;
     int numLines;
     int size[2];
     int cpp;
@@ -298,15 +288,19 @@ ImgXpmGetData(interp, masterPtr)
 
     if (masterPtr->id != NULL) {
 	data = ImgXpmGetDataFromId(interp, masterPtr->id);
-    } else if (masterPtr->fileString != NULL) {
+	isAllocated = 0;
+    }
+    else if (masterPtr->fileString != NULL) {
 	data = ImgXpmGetDataFromFile(interp, masterPtr->fileString, &numLines);
 	isAllocated = 1;
-    } else if (masterPtr->dataString != NULL) {
+    }
+    else if (masterPtr->dataString != NULL) {
 	data = ImgXpmGetDataFromString(interp,masterPtr->dataString,&numLines);
 	isAllocated = 1;
-    } else {
+    }
+    else {
 	/* Should have been enforced by ImgXpmConfigureMaster() */
-	Tcl_Panic("ImgXpmGetData(): -data, -file and -id are all NULL");
+	panic("ImgXpmGetData(): -data, -file and -id are all NULL");
     }
 
     if (data == NULL) {
@@ -319,11 +313,20 @@ ImgXpmGetData(interp, masterPtr)
 	code = TCL_ERROR; goto done;
     }
 
-    if ((listArgc < 4) /* file format error */
-	    || (Tcl_GetInt(interp, listArgv[0], &size[0]) != TCL_OK)
-	    || (Tcl_GetInt(interp, listArgv[1], &size[1]) != TCL_OK)
-	    || (Tcl_GetInt(interp, listArgv[2], &ncolors) != TCL_OK)
-	    || (Tcl_GetInt(interp, listArgv[3], &cpp) != TCL_OK)) {
+    if (listArgc < 4) {	/* file format error */
+	code = TCL_ERROR; goto done;
+    }
+
+    if (Tcl_GetInt(interp, listArgv[0], &size[0]) != TCL_OK) {
+	code = TCL_ERROR; goto done;
+    }
+    if (Tcl_GetInt(interp, listArgv[1], &size[1]) != TCL_OK) {
+	code = TCL_ERROR; goto done;
+    }
+    if (Tcl_GetInt(interp, listArgv[2], &ncolors) != TCL_OK) {
+	code = TCL_ERROR; goto done;
+    }
+    if (Tcl_GetInt(interp, listArgv[3], &cpp) != TCL_OK) {
 	code = TCL_ERROR; goto done;
     }
 
@@ -359,13 +362,13 @@ ImgXpmGetData(interp, masterPtr)
     if (listArgv) {
 	ckfree((char*)listArgv);
     }
-
+		   
     return code;
 }
 
 static char ** ImgXpmGetDataFromId(interp, id)
     Tcl_Interp * interp;
-    CONST84 char * id;
+    char * id;
 {
     Tcl_HashEntry * hashPtr;
 
@@ -518,59 +521,40 @@ static char ** ImgXpmGetDataFromFile(interp, fileName, numLines_return)
     char * fileName;
     int * numLines_return;
 {
-    FILE * fd = NULL;
-    int size, n;
+    int fileId, size;
     char ** data;
+    struct stat statBuf;
     char *cmdBuffer = NULL;
     Tcl_DString buffer;			/* initialized by Tcl_TildeSubst */
 
-    fileName = Tcl_TranslateFileName(interp, fileName, &buffer);
+    fileName = Tcl_TildeSubst(interp, fileName, &buffer);
     if (fileName == NULL) {
 	goto error;
     }
 
-    /*
-     * Open the file and find its size
-     */
-
-    fd = fopen(fileName, "r");
-    if (fd == NULL) {
+    fileId = open(fileName, O_RDONLY, 0);
+    if (fileId < 0) {
 	Tcl_AppendResult(interp, "couldn't read file \"", fileName,
 		"\": ", Tcl_PosixError(interp), (char *) NULL);
 	goto error;
     }
-    if (fseek(fd, 0, SEEK_END) < 0) {
-	Tcl_AppendResult(interp, "couldn't fseek file \"", fileName,
+    if (fstat(fileId, &statBuf) == -1) {
+	Tcl_AppendResult(interp, "couldn't stat file \"", fileName,
 		"\": ", Tcl_PosixError(interp), (char *) NULL);
+	close(fileId);
 	goto error;
     }
-    size = ftell(fd);
+    cmdBuffer = (char *) ckalloc((unsigned) statBuf.st_size+1);
+    size = read(fileId, cmdBuffer, (size_t) statBuf.st_size);
     if (size < 0) {
-	Tcl_AppendResult(interp, "couldn't ftell file \"", fileName,
-		"\": ", Tcl_PosixError(interp), (char *) NULL);
-	goto error;
-    }
-    if (fseek(fd, 0, SEEK_SET) < 0) {
-	Tcl_AppendResult(interp, "couldn't fseek file \"", fileName,
-		"\": ", Tcl_PosixError(interp), (char *) NULL);
-	goto error;
-    }
-
-    /*
-     * Read the entire file
-     */
-
-    cmdBuffer = (char *) ckalloc((unsigned) size + 1);
-    n = fread(cmdBuffer, 1, (size_t) size, fd);
-    if (size != n) {
 	Tcl_AppendResult(interp, "error in reading file \"", fileName,
 		"\": ", Tcl_PosixError(interp), (char *) NULL);
+	close(fileId);
 	goto error;
     }
-    if (fclose(fd) != 0) {
+    if (close(fileId) != 0) {
 	Tcl_AppendResult(interp, "error closing file \"", fileName,
 		"\": ", Tcl_PosixError(interp), (char *) NULL);
-        fd = NULL;
 	goto error;
     }
     cmdBuffer[size] = 0;
@@ -581,9 +565,6 @@ static char ** ImgXpmGetDataFromFile(interp, fileName, numLines_return)
     return data;
 
   error:
-    if (fd != NULL) {
-        fclose(fd);
-    }
     if (cmdBuffer != NULL) {
 	ckfree(cmdBuffer);
     }
@@ -745,7 +726,7 @@ ImgXpmGetPixmapFromData(interp, masterPtr, instancePtr)
 	if (masterPtr->cpp == 1) {
 	    colors[i].c = 0;
 	} else {
-	    colors[i].cstring = (char*)ckalloc((unsigned) masterPtr->cpp);
+	    colors[i].cstring = (char*)ckalloc(masterPtr->cpp);
 	    colors[i].cstring[0] = 0;
 	}
     }
@@ -957,15 +938,16 @@ ImgXpmCmd(clientData, interp, argc, argv)
     ClientData clientData;	/* Information about button widget. */
     Tcl_Interp *interp;		/* Current interpreter. */
     int argc;			/* Number of arguments. */
-    CONST84 char **argv;	/* Argument strings. */
+    char **argv;		/* Argument strings. */
 {
     PixmapMaster *masterPtr = (PixmapMaster *) clientData;
     int c, code;
     size_t length;
 
     if (argc < 2) {
-	Tcl_AppendResult(interp, "wrong # args: should be \"",
-		argv[0], " option ?arg arg ...?\"", NULL);
+	sprintf(interp->result,
+	    "wrong # args: should be \"%.50s option ?arg arg ...?\"",
+	    argv[0]);
 	return TCL_ERROR;
     }
     c = argv[1][0];
