@@ -1,25 +1,13 @@
-# -*- mode: TCL; fill-column: 75; tab-width: 8; coding: iso-latin-1-unix -*-
-#
-#	$Id: EFileDlg.tcl,v 1.3 2002/01/24 09:13:58 idiscovery Exp $
-#
 # EFileDlg.tcl --
 #
 #	Implements the Extended File Selection Dialog widget.
 #
-# Copyright (c) 1993-1999 Ioi Kim Lam.
-# Copyright (c) 2000-2001 Tix Project Group.
+# Copyright (c) 1996, Expert Interface Technologies
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 #
 
-foreach fun {tkButtonInvoke} {
-    if {![llength [info commands $fun]]} {
-	tk::unsupported::ExposePrivateCommand $fun
-    }
-}
-unset fun
-
 tixWidgetClass tixExFileSelectDialog {
     -classname TixExFileSelectDialog
     -superclass tixDialogShell
@@ -41,8 +29,6 @@ proc tixExFileSelectDialog:ConstructWidget {w} {
     set data(w:fsbox) [tixExFileSelectBox $w.fsbox -dialog $w \
 	-command $data(-command)]
     pack $data(w:fsbox) -expand yes -fill both
-
-
 }
 
 proc tixExFileSelectDialog:config-command {w value} {
